@@ -111,13 +111,14 @@ sort_apps (gconstpointer a,
            gconstpointer b,
            gpointer      data)
 {
+  const char *empty = "";
   GAppInfo *info1 = G_APP_INFO (a);
   GAppInfo *info2 = G_APP_INFO (b);
 
   g_autofree char *s1 = g_utf8_casefold (g_app_info_get_name (info1), -1);
   g_autofree char *s2 = g_utf8_casefold (g_app_info_get_name (info2), -1);
 
-  return g_strcmp0 (s1, s2);
+  return g_utf8_collate (s1 ?: empty, s2 ?: empty);
 }
 
 
